From 5a914c54a0f1209b6123448a030985b30c730a75 Mon Sep 17 00:00:00 2001
From: Shugo Maeda <shugo@ruby-lang.org>
Date: Wed, 20 Oct 2010 22:20:22 +0900
Subject: [PATCH 32/51] supported classboxes in Module#module_{eval,exec}.
 added Module#classbox_{eval,exec} as aliases of module_{eval,exec}.

---
 insns.def       |    2 +-
 vm_eval.c       |    4 ++++
 vm_insnhelper.c |   12 +++++-------
 3 files changed, 10 insertions(+), 8 deletions(-)

diff --git a/insns.def b/insns.def
index 0ce5d55..b77ad7e 100644
--- a/insns.def
+++ b/insns.def
@@ -989,7 +989,7 @@ defineclass
 		  VM_FRAME_MAGIC_CLASS, klass, 0, (VALUE) GET_BLOCK_PTR(),
 		  class_iseq->iseq_encoded, GET_SP(), 0,
 		  class_iseq->local_size);
-    vm_import_classboxes(class_iseq->cref_stack, klass, define_type);
+    rb_vm_import_classboxes(class_iseq->cref_stack, klass);
     RESTORE_REGS();
 
     INC_VM_STATE_VERSION();
diff --git a/vm_eval.c b/vm_eval.c
index 2bca7db..0a916ca 100644
--- a/vm_eval.c
+++ b/vm_eval.c
@@ -1213,6 +1213,7 @@ yield_under(VALUE under, VALUE self, VALUE values)
     }
     cref = vm_cref_push(th, under, NOEX_PUBLIC, blockptr);
     cref->flags |= NODE_FL_CREF_PUSHED_BY_EVAL;
+    rb_vm_import_classboxes(cref, under);
 
     if (values == Qundef) {
 	return vm_yield_with_cref(th, 1, &self, cref);
@@ -1234,6 +1235,7 @@ eval_under(VALUE under, VALUE self, VALUE src, const char *file, int line)
     else {
 	SafeStringValue(src);
     }
+    rb_vm_import_classboxes(cref, under);
 
     return eval_string_with_cref(self, src, Qnil, cref, file, line);
 }
@@ -1781,8 +1783,10 @@ Init_vm_eval(void)
 
     rb_define_method(rb_cModule, "module_exec", rb_mod_module_exec, -1);
     rb_define_method(rb_cModule, "class_exec", rb_mod_module_exec, -1);
+    rb_define_method(rb_cModule, "classbox_exec", rb_mod_module_exec, -1);
     rb_define_method(rb_cModule, "module_eval", rb_mod_module_eval, -1);
     rb_define_method(rb_cModule, "class_eval", rb_mod_module_eval, -1);
+    rb_define_method(rb_cModule, "classbox_eval", rb_mod_module_eval, -1);
 
     rb_define_global_function("caller", rb_f_caller, -1);
 }
diff --git a/vm_insnhelper.c b/vm_insnhelper.c
index f46ec47..accdc91 100644
--- a/vm_insnhelper.c
+++ b/vm_insnhelper.c
@@ -1743,16 +1743,15 @@ vm_import_classbox_i(VALUE classbox, VALUE value, VALUE arg)
     return ST_CONTINUE;
 }
 
-static void
-vm_import_classboxes(NODE *cref, VALUE klass, int define_type)
+void
+rb_vm_import_classboxes(NODE *cref, VALUE klass)
 {
     ID id_imported_classboxes;
     VALUE imported_classboxes;
 
     CONST_ID(id_imported_classboxes, "__imported_classboxes__");
     imported_classboxes = rb_attr_get(klass, id_imported_classboxes);
-    switch (define_type) {
-    case 0:
+    if (TYPE(klass) == T_CLASS) {
 	if (NIL_P(imported_classboxes)) {
 	    VALUE super = rb_class_real(RCLASS_SUPER(klass));
 	    imported_classboxes = rb_attr_get(super, id_imported_classboxes);
@@ -1761,10 +1760,9 @@ vm_import_classboxes(NODE *cref, VALUE klass, int define_type)
 		rb_ivar_set(klass, id_imported_classboxes, imported_classboxes);
 	    }
 	}
-	break;
-    case 3:
+    }
+    else if (rb_obj_is_kind_of(klass, rb_cClassbox)) {
 	rb_import_classbox(cref, klass);
-	break;
     }
     if (!NIL_P(imported_classboxes)) {
 	rb_hash_foreach(imported_classboxes, vm_import_classbox_i,
-- 
1.7.0.4

